#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['xone']

package_data = \
{'': ['*']}

setup(name='xone',
      version='0.0.5',
      description='Frequently used functions for financial data analysis',
      author='alpha.xone',
      author_email='alpha.xone@outlook.com',
      url='https://github.com/alpha-xone/',
      packages=packages,
      package_data=package_data,
     )
